<?php

namespace Drupal\commerce_wallee\Services;

use Drupal\commerce_payment\Entity\PaymentMethodInterface;
use Drupal\commerce_wallee\Services\PaymentSdkInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\commerce_payment\Entity\PaymentGatewayInterface;
use Drupal\profile\Entity\ProfileInterface;

/**
 * Event handler for Wallee PHP SDK.
 */
class PaymentSdk implements PaymentSdkInterface {

  /**
   * @var EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * PaymentSdk constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * Create Wallee PHP SDK Client.
   *
   * @param $user_id
   * @param $secret
   *
   * @return \Wallee\Sdk\ApiClient
   */
  private function apiClient($user_id, $secret) {
    return new \Wallee\Sdk\ApiClient($user_id, $secret);
  }

  /**
   * Create Wallee line item.
   *
   * @param $order_id
   * @param $amount
   *
   * @return \Wallee\Sdk\Model\LineItemCreate
   */
  private function createLineItem($order_id, $amount) {
    $lineItem = new \Wallee\Sdk\Model\LineItemCreate();
    $lineItem->setName('Order ' . $order_id);
    $lineItem->setUniqueId('orderid-' . $order_id);
    $lineItem->setQuantity(1);
    $lineItem->setAmountIncludingTax($amount);
    $lineItem->setType(\Wallee\Sdk\Model\LineItemType::FEE);

    return $lineItem;
  }

  /**
   * Create Wallee address object.
   *
   * @param $organisation
   * @param $salutation
   * @param $family_name
   * @param $given_name
   * @param $street
   * @param $postal_code
   * @param $city
   * @param $postal_state
   * @param $country
   * @param $mail
   * @param $phone
   *
   * @return \Wallee\Sdk\Model\Address
   */
  public function createAddress($organisation, $salutation, $family_name, $given_name, $street, $postal_code, $city, $postal_state, $country, $mail, $phone) {
    $profileAddress = new \Wallee\Sdk\Model\Address();

    if ($organisation != '') {
      $profileAddress->setOrganizationName($organisation);
    }
    if ($salutation != '') {
      $profileAddress->setSalutation($salutation);
    }
    if ($family_name != '') {
      $profileAddress->setFamilyName($family_name);
    }
    if ($given_name != '') {
      $profileAddress->setGivenName($given_name);
    }
    if ($street != '') {
      $profileAddress->setStreet($street);
    }
    if ($postal_code != '') {
      $profileAddress->setPostCode($postal_code);
    }
    if ($city != '') {
      $profileAddress->setCity($city);
    }
    if ($postal_state != '') {
      $profileAddress->setPostalState($postal_state);
    }
    if ($country != '') {
      $profileAddress->setCountry($country);
    }
    if ($mail != '') {
      $profileAddress->setEmailAddress($mail);
    }
    if ($phone != '') {
      $profileAddress->setPhoneNumber($phone);
    }

    return $profileAddress;
  }

  /**
   * Create Wallee transaction.
   *
   * @param $user_id
   * @param $secret
   * @param $space_id
   * @param $return_url
   * @param $cancel_url
   * @param $currency
   * @param $order_id
   * @param $amount
   * @param $billingAddress
   * @param $shippingAddress
   * @param string $environment
   * @param array $pluginConfiguration
   *
   * @return array
   * @throws \Wallee\Sdk\ApiException
   * @throws \Wallee\Sdk\Http\ConnectionException
   * @throws \Wallee\Sdk\VersioningException
   */
  public function createTransaction($user_id, $secret, $space_id, $return_url, $cancel_url, $currency, $order_id, $amount, $billingAddress, $shippingAddress, $environment = 'PREVIEW', $pluginConfiguration = []) {
    // Set user language.
    $language = \Drupal::languageManager()->getCurrentLanguage()->getId();
    $language_string = strtolower($language) . '-' . strtoupper($language);

    $transactionPayload = new \Wallee\Sdk\Model\TransactionCreate();
    $transactionPayload->setMerchantReference($order_id);
    $transactionPayload->setLanguage($language_string);
    $transactionPayload->setSuccessUrl($return_url);
    $transactionPayload->setFailedUrl($cancel_url);
    $transactionPayload->setEnvironment($environment);
    if ($billingAddress != '') {
      $transactionPayload->setBillingAddress($billingAddress);
    }
    if ($shippingAddress != '') {
      $transactionPayload->setShippingAddress($shippingAddress);
    }
    $transactionPayload->setCurrency($currency);
    $transactionPayload->setLineItems([$this->createLineItem($order_id, $amount)]);
    $transactionPayload->setAutoConfirmationEnabled(true);

    if (isset($pluginConfiguration['payment_configuration']) AND $pluginConfiguration['payment_configuration'] != 'all') {
      $transactionPayload->setAllowedPaymentMethodConfigurations($pluginConfiguration['payment_configuration']);
    }

    // Get user settings.
    if (\Drupal::currentUser()->id() != 0) {
      // Get user temp store.
      $tempstore = \Drupal::service('tempstore.private');
      $store = $tempstore->get('commerce_wallee');
      $save_payment = $store->get('save_payment', 0);
      $uid = $store->get('uid', 0);
      if ($save_payment == TRUE) {
        // Force token generation on wallee.
        $transactionPayload->setTokenizationMode(\Wallee\Sdk\Model\TokenizationMode::FORCE_CREATION_WITH_ONE_CLICK_PAYMENT);
        $transactionPayload->setCustomerEmailAddress($transactionPayload->getBillingAddress()->getEmailAddress());
        $transactionPayload->setCustomerId($uid);
      }
    }

    // Allow alter transaction over hook.
    \Drupal::moduleHandler()->invokeAll('commerce_wallee_alter_transaction', [&$transactionPayload, $order_id]);

    // Create transaction.
    $client = $this->apiClient($user_id, $secret);
    $transaction = $client->getTransactionService()->create($space_id, $transactionPayload);

    return [
      'transaction_id' => $transaction->getId(),
      'redirect_url' => $client->getTransactionPaymentPageService()->paymentPageUrl($space_id, $transaction->getId()),
    ];
  }

  /**
   * Create Wallee transaction with no user interaction.
   *
   * @param $user_id
   * @param $secret
   * @param $space_id
   * @param $currency
   * @param $order
   * @param $amount
   * @param $billingAddress
   * @param $shippingAddress
   * @param $token
   * @param string $environment
   *
   * @return array
   * @throws \Wallee\Sdk\ApiException
   * @throws \Wallee\Sdk\Http\ConnectionException
   * @throws \Wallee\Sdk\VersioningException
   */
  public function createTransactionNoUserInteraction($user_id, $secret, $space_id, $currency, $order, $amount, $billingAddress, $shippingAddress, $token, $environment = 'PREVIEW') {

    $transactionPayload = new \Wallee\Sdk\Model\TransactionCreate();
    $transactionPayload->setMerchantReference($order->id());
    $transactionPayload->setEnvironment($environment);
    if ($billingAddress != '') {
      $billing = $billingAddress->get('address')->first();
      $address = $this->createAddress(
        $billing->getOrganization(),
        '',
        $billing->getFamilyName(),
        $billing->getGivenName(),
        $billing->getAddressLine1(),
        $billing->getPostalCode(),
        $billing->getLocality(),
        '',
        $billing->getCountryCode(),
        $order->getEmail(),
        ''
      );

      $transactionPayload->setBillingAddress($address);
    }
    if ($shippingAddress != '') {
      $shipping = $shippingAddress->get('address')->first();
      $address = $this->createAddress(
        $shipping->getOrganization(),
        '',
        $shipping->getFamilyName(),
        $shipping->getGivenName(),
        $shipping->getAddressLine1(),
        $shipping->getPostalCode(),
        $shipping->getLocality(),
        '',
        $shipping->getCountryCode(),
        $order->getEmail(),
        ''
      );

      $transactionPayload->setShippingAddress($address);
    }
    $transactionPayload->setCurrency($currency);
    $transactionPayload->setLineItems([$this->createLineItem($order->id(), $amount)]);
    $transactionPayload->setAutoConfirmationEnabled(true);

    // Set token to transaction.
    $transactionPayload->setToken($token);

    // Create transaction.
    $client = $this->apiClient($user_id, $secret);
    $transaction = $client->getTransactionService()->create($space_id, $transactionPayload);
    $transaction_service = new \Wallee\Sdk\Service\TransactionService($client);
    $transaction_return = $transaction_service->processWithoutUserInteraction($space_id, $transaction->getId());

    return [
      'id' => $transaction_return->getId(),
      'authorization_amount' => $transaction_return->getAuthorizationAmount(),
      'authorization_environment' => $transaction_return->getAuthorizationEnvironment(),
      'authorized_on' => $transaction_return->getAuthorizedOn(),
      'completed_amount' => $transaction_return->getCompletedAmount(),
      'completed_on' => $transaction_return->getCompletedOn(),
      'currency' => $transaction_return->getCurrency(),
      'state' => $transaction_return->getState(),
      'token' => $transaction_return->getToken(),
    ];
  }

  /**
   * Transform Wallee transaction state to drupal commerce state.
   *
   * @param $wallee_state
   *
   * @return string|null
   */
  public function transformTransactionState($wallee_state) {
    // State is authorized.
    $commerce_sate = NULL;
    if ($wallee_state == 'VOIDED') {
      $commerce_sate = 'void';
    }
    if ($wallee_state == 'AUTHORIZED') {
      $commerce_sate = 'authorize';
    }
    if ($wallee_state == 'COMPLETED') {
      $commerce_sate = 'completed';
    }
    if ($wallee_state == 'FULFILL') {
      $commerce_sate = 'completed';
    }

    return $commerce_sate;
  }

  /**
   * Read transaction.
   *
   * @param $user_id
   * @param $secret
   * @param $space_id
   * @param $transaction_id
   *
   * @return array
   * @throws \Wallee\Sdk\ApiException
   * @throws \Wallee\Sdk\Http\ConnectionException
   * @throws \Wallee\Sdk\VersioningException
   */
  public function readTransaction($user_id, $secret, $space_id, $transaction_id) {
    $client = $this->apiClient($user_id, $secret);

    $transaction = $client->getTransactionService()->read($space_id, $transaction_id);

    return [
      'id' => $transaction_id,
      'authorization_amount' => $transaction->getAuthorizationAmount(),
      'authorization_environment' => $transaction->getAuthorizationEnvironment(),
      'authorized_on' => $transaction->getAuthorizedOn(),
      'completed_amount' => $transaction->getCompletedAmount(),
      'completed_on' => $transaction->getCompletedOn(),
      'currency' => $transaction->getCurrency(),
      'state' => $transaction->getState(),
      'token' => $transaction->getToken(),
    ];
  }

  /**
   * Create refund for transaction.
   */
  public function createRefund($user_id, $secret, $space_id, $amount, $remote_id) {
    // Create refund object.
    $trans = new \Wallee\Sdk\Model\Transaction();
    $trans->setId($remote_id);

    $refund = new \Wallee\Sdk\Model\Refund();
    $refund->setAmount($amount);
    $refund->setExternalId(time());
    $refund->setMerchantReference('commerce_wallee');
    $refund->setTransaction($trans);
    $refund->setType('MERCHANT_INITIATED_ONLINE');

    // Setup API client and call refund.
    $client = $this->apiClient($user_id, $secret);
    return $client->getRefundService()->refund($space_id, $refund);
  }

  /**
   * Create Payment method with user data.
   *
   * @param \Drupal\commerce_payment\Entity\PaymentGatewayInterface $commerce_payment_gateway
   * @param $remote_id
   * @param $user_id
   * @param \Drupal\profile\Entity\ProfileInterface $billing_profile
   *
   * @return \Drupal\Core\Entity\EntityInterface|\Drupal\Core\Entity\EntityInterface[]|false|null
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   * @throws \Wallee\Sdk\ApiException
   * @throws \Wallee\Sdk\Http\ConnectionException
   * @throws \Wallee\Sdk\VersioningException
   */
  public function createPaymentMethod(PaymentGatewayInterface $commerce_payment_gateway, $remote_id, $user_id, ProfileInterface $billing_profile) {
    // Owner.
    $owner = \Drupal\user\Entity\User::load($user_id);
    $owner_timezone = $owner->getTimeZone();

    // Get detail information from token version.
    $configuration = $commerce_payment_gateway->getPluginConfiguration();
    $client = $this->apiClient($configuration['user_id'], $configuration['secret']);
    $tokenVersionService = new \Wallee\Sdk\Service\TokenVersionService($client);
    $version = $tokenVersionService->activeVersion($configuration['space_id'], $remote_id);
    if ($version instanceof \Wallee\Sdk\Model\TokenVersion) {
      $card_endingnumber = $version->getName();
      $expires_timestamp = NULL;
      if ($version->getExpiresOn() instanceof \DateTime) {
        $expires_timestamp = $version->getExpiresOn()->setTimezone(new \DateTimeZone($owner_timezone));
        $expires_timestamp = $expires_timestamp->getTimestamp();
      }

      // Transform credit card type.
      $card_type = $this->mapCardType($version->getPaymentConnectorConfiguration()->getName());
      if ($card_type == FALSE) {
        // Set a default value.
        $card_type = 'visa';
      }

      // Check if remote id exists.
      $storage = $this->entityTypeManager->getStorage('commerce_payment_method')->loadByProperties(['remote_id' => $remote_id]);
      if (count($storage) > 0) {
        // Update existing.
        switch ($card_type) {
          case 'visa':
          case 'master':
            $storage = current($storage);
            $storage->expires->value = $expires_timestamp;
            $storage->card_type->value = $card_type;
            $storage->card_number->value = $card_endingnumber;
            $storage->card_exp_month->value = date('m', $expires_timestamp);
            $storage->card_exp_year->value = date('y', $expires_timestamp);
            break;
        }
      }
      else {
        // Create new one.
        switch ($card_type) {
          case 'visa':
          case 'mastercard':
            $values = [
              'type' => 'credit_card',
              'payment_gateway' => $commerce_payment_gateway,
              'remote_id' => $remote_id,
              'uid' => $user_id,
              'billing_profile' => $billing_profile,
              'reusable' => TRUE,
              'is_default' => TRUE,
              'expires' => $expires_timestamp,
              'card_type' => $card_type,
              'card_number' => $card_endingnumber,
              'card_exp_month' => date('m', $expires_timestamp),
              'card_exp_year' => date('y', $expires_timestamp),
            ];
            break;
          case 'paypal':
            // Extract mail address.
            $mail_address = '';
            $mail = mailparse_rfc822_parse_addresses($card_endingnumber);
            if (isset($mail[0]['address'])) {
              $mail_address = $mail[0]['address'];
            }

            $values = [
              'type' => 'paypal',
              'payment_gateway' => $commerce_payment_gateway,
              'remote_id' => $remote_id,
              'uid' => $user_id,
              'billing_profile' => $billing_profile,
              'reusable' => TRUE,
              'is_default' => TRUE,
              'paypal_mail' => $mail_address,
            ];
            break;
        }

        $storage = $this->entityTypeManager->getStorage('commerce_payment_method')->create($values);
      }
      $storage->save();
      return $storage;
    }
    else {
      return NULL;
    }
  }

  /**
   * Delete Payment method on wallee.
   *
   * @param \Drupal\commerce_payment\Entity\PaymentMethodInterface $payment_method
   *
   * @throws \Wallee\Sdk\ApiException
   * @throws \Wallee\Sdk\Http\ConnectionException
   * @throws \Wallee\Sdk\VersioningException
   */
  public function deletePaymentMethod(PaymentMethodInterface $payment_method) {
    // Check if gateway is paymentgateway entity.
    if ($payment_method->payment_gateway->entity instanceof \Drupal\commerce_payment\Entity\PaymentGateway) {
      $configuration = $payment_method->payment_gateway->entity->getPluginConfiguration();
      $client = $this->apiClient($configuration['user_id'], $configuration['secret']);
      return $client->getTokenService()->delete($configuration['space_id'], $payment_method->getRemoteId());
    }
    return FALSE;
  }

  /**
   * Update Payment method on wallee.
   *
   * @param \Drupal\commerce_payment\Entity\PaymentMethodInterface $payment_method
   *
   * @throws \Wallee\Sdk\ApiException
   * @throws \Wallee\Sdk\Http\ConnectionException
   * @throws \Wallee\Sdk\VersioningException
   */
  public function updatePaymentMethod(PaymentMethodInterface $payment_method, $status = '') {
    // Extract full URL.
    $current_page = parse_url((isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? 'https' : 'http') . '://' . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI']);
    $url = $current_page['scheme'] . '://' . $current_page['host'] . $current_page['path'];

    // Check if gateway is paymentgateway entity.
    if ($payment_method->payment_gateway->entity instanceof \Drupal\commerce_payment\Entity\PaymentGateway) {
      // Create API Client.
      $pluginConfiguration = $payment_method->payment_gateway->entity->getPluginConfiguration();
      $client = $this->apiClient($pluginConfiguration['user_id'], $pluginConfiguration['secret']);

      if ($status == '') {
        $tokenService = new \Wallee\Sdk\Service\TokenService($client);
        $transactionPayload = $tokenService->createTransactionForTokenUpdate($pluginConfiguration['space_id'], $payment_method->getRemoteId());
        $pendingTransaction = new \Wallee\Sdk\Model\TransactionPending();

        if (isset($pluginConfiguration['payment_configuration']) and $pluginConfiguration['payment_configuration'] != 'all') {
          $pendingTransaction->setAllowedPaymentMethodConfigurations($pluginConfiguration['payment_configuration']);
        }

        $pendingTransaction->setId($transactionPayload->getId());
        $pendingTransaction->setVersion($transactionPayload->getVersion());
        $pendingTransaction->setSuccessUrl($url . '?wallee_update=success');
        $pendingTransaction->setFailedUrl($url . '?wallee_update=failed');
        $transaction = $client->getTransactionService()->update($pluginConfiguration['space_id'], $pendingTransaction);
        $redirectionUrl = $client->getTransactionPaymentPageService()->paymentPageUrl($pluginConfiguration['space_id'], $transaction->getId());
        header('Location: ' . $redirectionUrl);
        exit();
      }

      if ($status == 'success') {
        \Drupal::messenger()->addMessage(t('The information will be updated soon.'));

        $current_user = \Drupal::currentUser();
        $response = new \Symfony\Component\HttpFoundation\RedirectResponse('/user/' . $current_user->id() . '/payment-methods');
        $response->send();
      }

      if ($status == 'failed') {
        \Drupal::messenger()->addError(t('Update failed, please try again later.'));

        $current_user = \Drupal::currentUser();
        $response = new \Symfony\Component\HttpFoundation\RedirectResponse('/user/' . $current_user->id() . '/payment-methods');
        $response->send();
      }
    }
  }

  /**
   * Webhook for Token update.
   *
   * @param \Drupal\commerce_payment\Entity\PaymentGateway $paymentGateway
   * @param $data
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Wallee\Sdk\ApiException
   * @throws \Wallee\Sdk\Http\ConnectionException
   * @throws \Wallee\Sdk\VersioningException
   */
  public function webhookToken(\Drupal\commerce_payment\Entity\PaymentGateway $paymentGateway, $data) {
    // Get configuration and create client.
    $configuration =  $paymentGateway->getPluginConfiguration();
    $client = $this->apiClient($configuration['user_id'], $configuration['secret']);

    // Get token entity.
    $payment_method = $this->entityTypeManager->getStorage('commerce_payment_method')->loadByProperties(['remote_id' => $data['entityId'], 'payment_gateway.target_id' => $paymentGateway->id()]);
    if (count($payment_method) > 0 AND current($payment_method) instanceof \Drupal\commerce_payment\Entity\PaymentMethod) {
      // Get token with entity id.
      $payment_method = current($payment_method);
      $tokenService = new \Wallee\Sdk\Service\TokenService($client);
      $token = $tokenService->read($configuration['space_id'], $data['entityId']);
      if ($token instanceof \Wallee\Sdk\Model\Token) {
        // Check token state: DELETE.
        if ($token->getState() == 'DELETED') {
          $payment_method->delete();
        }
      }
    }
  }

  /**
   * Webhook for Token Version update.
   *
   * @param \Drupal\commerce_payment\Entity\PaymentGateway $paymentGateway
   * @param $data
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Wallee\Sdk\ApiException
   * @throws \Wallee\Sdk\Http\ConnectionException
   * @throws \Wallee\Sdk\VersioningException
   */
  public function webhookTokenVersion(\Drupal\commerce_payment\Entity\PaymentGateway $paymentGateway, $data) {
    // Get configuration and create client.
    $configuration =  $paymentGateway->getPluginConfiguration();
    $client = $this->apiClient($configuration['user_id'], $configuration['secret']);

    // Get token version details.
    $tokenVersionService = new \Wallee\Sdk\Service\TokenVersionService($client);
    $version = $tokenVersionService->read($configuration['space_id'], $data['entityId']);
    if ($version instanceof \Wallee\Sdk\Model\TokenVersion) {
      // Get token entity.
      $remote_id =$version->getToken()->getId();
      $card_endingnumber =$version->getName();

      if ($version->getPaymentConnectorConfiguration() == NULL) {
        return FALSE;
      }

      $card_type = $this->mapCardType($version->getPaymentConnectorConfiguration()->getName());
      $payment_method = $this->entityTypeManager->getStorage('commerce_payment_method')->loadByProperties(['remote_id' => $remote_id, 'payment_gateway.target_id' => $paymentGateway->id()]);
      if (count($payment_method) > 0 AND current($payment_method) instanceof \Drupal\commerce_payment\Entity\PaymentMethod) {
        // Get token with entity id.
        $payment_method = current($payment_method);
        if ($card_type == FALSE) {
          // Set a default value.
          $card_type = 'visa';
        }

        switch ($card_type) {
          case 'visa':
          case 'mastercard':
            $owner = $payment_method->uid->entity;
            $owner_timezone = $owner->getTimeZone();

            $expires_timestamp = '';
            if ($version->getExpiresOn() != '') {
              $expires_timestamp = $version->getExpiresOn()->setTimezone(new \DateTimeZone($owner_timezone));
              $expires_timestamp = $expires_timestamp->getTimestamp();
            }

            $payment_method->expires->value = $expires_timestamp;
            $payment_method->card_type->value = $card_type;
            $payment_method->card_number->value = $card_endingnumber;
            $payment_method->card_exp_month->value = date('m', $expires_timestamp);
            $payment_method->card_exp_year->value = date('y', $expires_timestamp);
            break;
        }

        // Update existing token with remote information.
        $payment_method->save();
      }
    }
  }

  /**
   * Webhook for Transaction update.
   *
   * @param \Drupal\commerce_payment\Entity\PaymentGateway $paymentGateway
   * @param $data
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function webhookTransaction(\Drupal\commerce_payment\Entity\PaymentGateway $paymentGateway, $data) {
    // Get configuration and create client.
    $configuration =  $paymentGateway->getPluginConfiguration();

    // Get token entity.
    $payment_storage = $this->entityTypeManager->getStorage('commerce_payment');
    $payments = $payment_storage->loadByProperties(['remote_id' => $data['entityId'], 'payment_gateway' => $paymentGateway->id()]);
    $payment = reset($payments);
    if ($payment instanceof PaymentInterface) {
      $transation = $this->readTransaction($configuration['user_id'], $configuration['secret'], $configuration['space_id'], $data['entityId']);

      // Check state of transaction.
      // More infos https://app-wallee.com/de-ch/doc/payment/transaction-process.
      if (isset($transation['state'])) {
        // Transform wallee state.
        $state = $this->transformTransactionState($transation['state']);

        // Update payment.
        if ($state != NULL) {

          // This does not need to update the order, but use the loadForUpdate()
          // method of the order storage, this locking system requires
          // https://www.drupal.org/project/commerce/issues/3043180.
          $order_storage = $this->entityTypeManager->getStorage('commerce_order');
          if (\method_exists($order_storage, 'loadForUpdate')) {
            $order_storage->loadForUpdate($payment->getOrderId());

            // Reload the payment entity.
            $payment_storage->resetCache([$payment->id()]);
            $payment = $payment_storage->load($payment->id());
          }

          $payment->state->value = $state;
          $payment->remote_state->value = $state;
          $payment->completed->value = time();
          $payment->save();

          // The order storage does not have a public method to release the
          // lock. The current implementation uss the lock backend with
          // a given lock id.
          if (\method_exists($order_storage, 'loadForUpdate')) {
            $lock_id = 'commerce_order_update:' . $payment->getOrderId();
            \Drupal::lock()->release($lock_id);
          }
        }
      }
    }
  }

  /**
   * Maps the credit card type to a Commerce credit card type.
   *
   * @param $card_type
   *
   * @return mixed|string
   */
  public function mapCardType($wallee_card_type) {
    $card_type = '';
    if (strpos(strtolower($wallee_card_type), 'visa') !== false) {
      $card_type = 'visa';
    }
    if (strpos(strtolower($wallee_card_type), 'mastercard') !== false) {
      $card_type = 'mastercard';
    }
    if (strpos(strtolower($wallee_card_type), 'paypal') !== false) {
      $card_type = 'paypal';
    }

    return $card_type;
  }
}
